// This MFC Samples source code demonstrates using MFC Microsoft Office Fluent User Interface 
// (the "Fluent UI") and is provided only as referential material to supplement the 
// Microsoft Foundation Classes Reference and related electronic documentation 
// included with the MFC C++ library software.  
// License terms to copy, use or distribute the Fluent UI are available separately.  
// To learn more about our Fluent UI licensing program, please visit 
// http://go.microsoft.com/fwlink/?LinkId=238214.
//
// Copyright (C) Microsoft Corporation
// All rights reserved.

// CodeEditorView.cpp : implementation of the CCodeEditorView class
//

#include "stdafx.h"
// SHARED_HANDLERS can be defined in an ATL project implementing preview, thumbnail
// and search filter handlers and allows sharing of document code with that project.
#ifndef SHARED_HANDLERS
#include "CodeEditor.h"
#endif

#include "CodeEditorDoc.h"
#include "CntrItem.h"
#include "resource.h"
#include "CodeEditorView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CCodeEditorView

IMPLEMENT_DYNCREATE(CCodeEditorView, CRichEditView)

BEGIN_MESSAGE_MAP(CCodeEditorView, CRichEditView)
	ON_WM_DESTROY()
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, &CRichEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, &CRichEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, &CCodeEditorView::OnFilePrintPreview)
	ON_WM_CONTEXTMENU()
	ON_WM_RBUTTONUP()
	ON_NOTIFY_REFLECT(EN_SELCHANGE, &CCodeEditorView::OnEnSelchange)
	ON_CONTROL_REFLECT(EN_UPDATE, &CCodeEditorView::OnEnUpdate)
	ON_MESSAGE(WM_RTF_NUMBER_UPDATE, &CCodeEditorView::OnRtfNumberUpdate)
	ON_WM_SIZE()
END_MESSAGE_MAP()

// CCodeEditorView construction/destruction

CCodeEditorView::CCodeEditorView()
{
	// TODO: add construction code here

}

CCodeEditorView::~CCodeEditorView()
{
}

BOOL CCodeEditorView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CRichEditView::PreCreateWindow(cs);
}

LRESULT CCodeEditorView::OnRtfNumberUpdate(WPARAM wParam, LPARAM lParam)
{

	CRichEditCtrl& EditCtrl = this->GetRichEditCtrl();

	// rich editor control's client rect
	CRect rect; EditCtrl.GetClientRect(&rect);

	rect.right = RTF_NUMBER_RIGHT_MARGIN;

	CMemDC MemDC(*EditCtrl.GetDC(), &rect);

	CDC *pDC = &MemDC.GetDC();

	pDC->FillSolidRect(&rect, RTF_MARGIN_BKCOLOR);

	int bkmode = pDC->SetBkMode(TRANSPARENT);
	COLORREF clr = pDC->SetTextColor(RTF_MARGIN_TXCOLOR);

	int height = rect.Height();
	int first_visible_line = EditCtrl.GetFirstVisibleLine();
	int line_count = EditCtrl.GetLineCount();

	// point of current character
	CPoint cpt = EditCtrl.GetCharPos(EditCtrl.LineIndex(-1));
	CPoint pt = EditCtrl.GetCharPos(EditCtrl.LineIndex(first_visible_line));

	CRect crect = rect; CString msg;

	for (int i = first_visible_line; i < line_count; )
	{
		crect.top = pt.y + 4;
		crect.bottom = pt.y + 22;

		if (crect.top > rect.bottom) break;

		msg.Format(L"%ld", i + 1);
		
		if (cpt.y == pt.y)
			pDC->SetTextColor(RTF_MARGIN_SLCOLOR);

		pDC->DrawText(msg, &crect, DT_CENTER | DT_VCENTER);

		if (cpt.y == pt.y)
			pDC->SetTextColor(RTF_MARGIN_TXCOLOR);

		pt = EditCtrl.GetCharPos(EditCtrl.LineIndex(++i));

		if (pt.y >= height) break;
	}

	pDC->SetBkMode(bkmode);
	pDC->SetTextColor(clr);
	
	return TRUE;
}


void CCodeEditorView::OnInitialUpdate()
{
	CRichEditView::OnInitialUpdate();

	CRichEditCtrl& EditCtrl = this->GetRichEditCtrl();

	EditCtrl.SetEventMask(ENM_SELCHANGE | ENM_UPDATE);


	// Set the printing margins (720 twips = 1/2 inch)
	SetMargins(CRect(720, 720, 720, 720));
}


// CCodeEditorView printing


void CCodeEditorView::OnFilePrintPreview()
{
#ifndef SHARED_HANDLERS
	AFXPrintPreview(this);
#endif
}

BOOL CCodeEditorView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}


void CCodeEditorView::OnDestroy()
{
	// Deactivate the item on destruction; this is important
	// when a splitter view is being used
   COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
   if (pActiveItem != NULL && pActiveItem->GetActiveView() == this)
   {
      pActiveItem->Deactivate();
      ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
   }
   CRichEditView::OnDestroy();
}


void CCodeEditorView::OnRButtonUp(UINT /* nFlags */, CPoint point)
{
	ClientToScreen(&point);
	OnContextMenu(this, point);
}

void CCodeEditorView::OnContextMenu(CWnd* /* pWnd */, CPoint point)
{
#ifndef SHARED_HANDLERS
	theApp.GetContextMenuManager()->ShowPopupMenu(IDR_POPUP_EDIT, point.x, point.y, this, TRUE);
#endif
}


// CCodeEditorView diagnostics

#ifdef _DEBUG
void CCodeEditorView::AssertValid() const
{
	CRichEditView::AssertValid();
}

void CCodeEditorView::Dump(CDumpContext& dc) const
{
	CRichEditView::Dump(dc);
}

CCodeEditorDoc* CCodeEditorView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCodeEditorDoc)));
	return (CCodeEditorDoc*)m_pDocument;
}
#endif //_DEBUG


// CCodeEditorView message handlers


void CCodeEditorView::OnEnSelchange(NMHDR *pNMHDR, LRESULT *pResult)
{
	SELCHANGE *pSelChange = reinterpret_cast<SELCHANGE *>(pNMHDR);
	// TODO:  The control will not send this notification unless you override the
	// CRichEditView::OnInitDialog() function to send the EM_SETEVENTMASK message
	// to the control with the ENM_SELCHANGE flag ORed into the lParam mask.

	// TODO:  Add your control notification handler code here

	this->PostMessageW(WM_RTF_NUMBER_UPDATE);

	*pResult = 0;
}


void CCodeEditorView::OnEnUpdate()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CRichEditView::OnInitDialog()
	// function to send the EM_SETEVENTMASK message to the control
	// with the ENM_UPDATE flag ORed into the lParam mask.

	// TODO:  Add your control notification handler code here
	this->PostMessageW(WM_RTF_NUMBER_UPDATE);

}


void CCodeEditorView::OnSize(UINT nType, int cx, int cy)
{
	CRichEditView::OnSize(nType, cx, cy);

	// TODO: Add your message handler code here

	if (::IsWindow(this->GetRichEditCtrl().GetSafeHwnd()))
	{
		CRect rect;
		this->GetClientRect(&rect);
		rect.left = RTF_NUMBER_RIGHT_MARGIN;

		this->GetRichEditCtrl().SetRect(&rect);
	}
}
